
package com.ora.intl.ime.arabichebrewim;
import java.awt.*;
import java.awt.event.*;
import java.awt.font.*;
import java.awt.im.*;
import java.awt.im.spi.*;
import java.io.*;
import java.text.*;
import java.util.*;

public class ArabicHebrewInputMethod implements InputMethod {
  private static Locale HEBREW = new Locale("iw", "IL");
  private static Locale ARABIC = new Locale("ar", "");
  private static Locale[] SUPPORTED_LOCALES = {ARABIC, HEBREW};

  // lookup table - shared by all instances of this input method
  private static Properties arabichebrewNames;

  // windows - shared by all instances of this input method
  private static Window statusWindow;

  // lookup information - per instance
  private Vector lookupCandidates;
  private int lookupCandidateCount;
  private LookupList lookupList;
  private int lookupSelection;

  // per-instance state
  private InputMethodContext inputMethodContext;
  private Locale locale;
  private boolean transliterated;
  private StringBuffer rawText;
  private String transliteratedText;

  public ArabicHebrewInputMethod() throws IOException {
    if (arabichebrewNames == null) {
      arabichebrewNames = loadProperties("names.properties");
    }
    rawText = new StringBuffer();
  }

  public void setInputMethodContext(InputMethodContext context) {
    inputMethodContext = context;
    if (statusWindow == null) {
      statusWindow = context.createInputMethodWindow(
                                                    "ArabicHebrew Input Method",
                                                    false);
      Label label = new Label();
      label.setBackground(Color.white);
      statusWindow.add(label);
      updateStatusWindow(locale);
      label.setSize(200, 50);
      statusWindow.pack();

      Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
      statusWindow.setLocation(d.width - statusWindow.getWidth(),
                               d.height - statusWindow.getHeight());
    }
  }

  public boolean setLocale(Locale locale) {
    for (int i = 0; i < SUPPORTED_LOCALES.length; i++) {
      if (locale.equals(SUPPORTED_LOCALES[i])) {
        this.locale = locale;
        if (statusWindow != null) {
          updateStatusWindow(locale);
        }
        return true;
      }
    }
    return false;
  }

  public Locale getLocale() {
    return locale;
  }

  public boolean isCompositionEnabled() {
    return true;
  }

  void updateStatusWindow(Locale locale) {
    Label label = (Label) statusWindow.getComponent(0);
    String localeName = locale == null ? "null" : locale.getDisplayName();
    label.setText(localeName + " IME Active");
  }

  public void setCharacterSubsets(Character.Subset[] subsets) {
  }

  public void setCompositionEnabled(boolean enable) {
    throw new UnsupportedOperationException();
  }

  public void notifyClientWindowChange(Rectangle location) {
  }

  public void dispatchEvent(AWTEvent event) {
    if (event.getID() == KeyEvent.KEY_TYPED) {
      KeyEvent e = (KeyEvent) event;
      if (handleCharacter(e.getKeyChar())) {
        e.consume();
      }
    }
  }

  public void activate() {
    if (!statusWindow.isVisible()) {
      statusWindow.setVisible(true);
    }
    updateStatusWindow(locale);
  }

  public void deactivate(boolean isTemporary) {
    closeLookupWindow();
  }

  public void hideWindows() {
    closeLookupWindow();
    statusWindow.hide();
  }

  public void removeNotify() {
  }

  public void endComposition() {
    if (rawText.length() != 0) {
      acceptSelection();
    }
    closeLookupWindow();
  }

  public void dispose() {
    closeLookupWindow();
  }

  public Object getControlObject() {
    return null;
  }

  public void reconvert() {
    throw new UnsupportedOperationException();
  }

  private Properties loadProperties(String fileName) throws IOException {
    Properties props = new Properties();
    InputStream stream = this.getClass().getResourceAsStream(fileName);
    props.load(stream);
    stream.close();
    return props;
  }

  private boolean handleCharacter(char ch) {
    // if the lookup window is active...
    if (lookupList != null) {
      if (ch == ' ') {
        if (++lookupSelection == lookupCandidateCount) {
          lookupSelection = 0;
        }
        selectCandidate(lookupSelection);
        return true;
      } else if (ch == '\n') {
        acceptSelection();
        closeLookupWindow();
        return true;
      }
      // otherwise, if the Latin letters have already been
      // transliterated to Arabic or Hebrew (transliterated state)
    } else if (transliterated) {
      if (ch == ' ') {
        showDerivedWords();
        return true;
      } else if (ch == '\n') {
        acceptSelection();
        return true;
      } else {
        Toolkit.getDefaultToolkit().beep();
        return true;
      }
      // initial state - we need to transliterate from Latin to Hebrew
    } else {
      if (ch == ' ') {
        transliterateLatinToRoot();
        return true;
      } else if (ch == '\n') {
        if (rawText.length() != 0) {
          acceptSelection();
          return true;
        }
      } else if (ch == '\b') {
        if (rawText.length() != 0) {
          rawText.setLength(rawText.length() - 1);
          sendText(false);
          return true;
        }
      // if a Latin letter is entered...
      } else if (Character.UnicodeBlock.of(ch) ==
        Character.UnicodeBlock.BASIC_LATIN && Character.isLetter(ch)) {
        rawText.append(ch);
        sendText(false);
        return true;
      // if we are already in the middle of composition, don't allow a
      // character other than a Latin letter to be entered.
      } else if (rawText.length() != 0) {
        Toolkit.getDefaultToolkit().beep();
        return true;
      }
    }
    return false;
  }

  private void transliterateLatinToRoot() {
    String lookupName;
    lookupName = rawText.toString();
    String localeLookupName = lookupName + "_" + locale;

    while (true) {
      transliteratedText = (String) arabichebrewNames.get(localeLookupName);
      if (transliteratedText != null) {
        break;
      }
      int index = localeLookupName.lastIndexOf("_");
      if (index == -1) {
        break;
      }
      localeLookupName = localeLookupName.substring(0, index);
    }

    if (transliteratedText != null) {
      transliterated = true;
      sendText(false);
    } else {
      Toolkit.getDefaultToolkit().beep();
    }
  }

  private void showDerivedWords() {
    String lookupName;
    lookupName = rawText.toString() + "_" + locale + ".properties";
    Properties lookupList;
    String lookupString;

    try {
      lookupList = loadProperties(lookupName);
    } catch (IOException e) {
      // no lookup information
      return;
    }

    lookupCandidates = new Vector();
    lookupCandidateCount = 1;

    while (true) {
      lookupString = (String)lookupList.getProperty(
                                         String.valueOf(lookupCandidateCount));
      if (lookupString == null) {
        lookupCandidateCount--;
        break;
      }

      lookupCandidates.add(lookupString);
      lookupCandidateCount++;
    }

    lookupSelection = 0;
    openLookupWindow();
  }

  private void acceptSelection() {
    sendText(true);
    rawText.setLength(0);
    transliteratedText = null;
    transliterated = false;
  }

  private void sendText(boolean committed) {
    String text;
    InputMethodHighlight highlight;
    int committedCharacterCount = 0;
    TextHitInfo caret = null;

    if (transliterated) {
      text = transliteratedText;
      highlight = InputMethodHighlight.SELECTED_CONVERTED_TEXT_HIGHLIGHT;
    } else {
      text = new String(rawText);
      highlight = InputMethodHighlight.SELECTED_RAW_TEXT_HIGHLIGHT;
    }

    AttributedString as = new AttributedString(text);
    if (committed) {
      committedCharacterCount = text.length();
    } else if (text.length() > 0) {
      as.addAttribute(TextAttribute.INPUT_METHOD_HIGHLIGHT, highlight);
      caret = TextHitInfo.leading(text.length());
    }

    inputMethodContext.dispatchInputMethodEvent(
                                    InputMethodEvent.INPUT_METHOD_TEXT_CHANGED,
                                    as.getIterator(),
                                    committedCharacterCount,
                                    caret,
                                    null);
  }

  void selectCandidate(int candidate) {
    lookupSelection = candidate;
    lookupList.selectCandidate(lookupSelection);
    transliteratedText = (String) lookupCandidates.elementAt(lookupSelection);
    sendText(false);
  }

  void openLookupWindow() {
    lookupList = new LookupList(this, inputMethodContext, lookupCandidates);
    lookupList.selectCandidate(lookupSelection);
  }

  void closeLookupWindow() {
    if (lookupList != null) {
      lookupList.setVisible(false);
      lookupList = null;
    }
  }
}

class LookupList extends Canvas {
  ArabicHebrewInputMethod inputMethod;
  InputMethodContext context;
  Window lookupWindow;
  Vector candidates; // list of candidates
  int candidateCount; // number of candidates in the vector
  int selected; // the currently select candidate
  final int INSIDE_INSET = 4;
  final int LINE_SPACING = 18;
  final int LINE_WIDTH = 200;

  LookupList(ArabicHebrewInputMethod inputMethod,
             InputMethodContext context,
             Vector candidates) {
    this.inputMethod = inputMethod;
    this.context = context;
    this.candidates = candidates;
    this.candidateCount = candidates.size();
    lookupWindow = context.createInputMethodWindow("Lookup Window", true);

    setFont(new Font("LucidaSans", Font.PLAIN, 18));
    setSize(LINE_WIDTH, candidateCount * LINE_SPACING + 2 * INSIDE_INSET);
    setForeground(Color.black);
    setBackground(Color.white);

    enableEvents(AWTEvent.KEY_EVENT_MASK);
    lookupWindow.add(this);
    lookupWindow.pack();
    updateWindowLocation();
    lookupWindow.setVisible(true);
  }

  /**
   * Positions the lookup window near (usually below) the
   * insertion point in the component where composition occurs.
   */
  private void updateWindowLocation() {
    Point windowLocation = new Point();
    Rectangle caretRect = context.getTextLocation(TextHitInfo.leading(0));
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension windowSize = lookupWindow.getSize();
    final int SPACING = 2;

    if (caretRect.x + windowSize.width > screenSize.width) {
      windowLocation.x = screenSize.width - windowSize.width;
    } else {
      windowLocation.x = caretRect.x;
    }

    if (caretRect.y + caretRect.height + SPACING +
      windowSize.height > screenSize.height) {
      windowLocation.y = caretRect.y - SPACING - windowSize.height;
    } else {
      windowLocation.y = caretRect.y + caretRect.height + SPACING;
    }
    lookupWindow.setLocation(windowLocation);
  }

  void selectCandidate(int candidate) {
    selected = candidate;
    repaint();
  }

  public void paint(Graphics g) {
    FontMetrics metrics = g.getFontMetrics();
    int descent = metrics.getDescent();
    int ascent = metrics.getAscent();
    String outString;
    Dimension size = getSize();

    g.fillRect(INSIDE_INSET / 2,
               LINE_SPACING*(selected + 1) + INSIDE_INSET-(descent + ascent +1),
               size.width - INSIDE_INSET,
               descent + ascent + 2);
    g.setColor(Color.black);

    for (int i = 0; i < candidateCount; i++) {
      outString = (i + 1) + ") " + (String)candidates.elementAt(i);
      if (i == selected) {
        g.setColor(Color.white);
      }
      g.drawString(outString,
                   (LINE_WIDTH - 4) - metrics.stringWidth(outString),
                   LINE_SPACING * (i + 1) + INSIDE_INSET - descent);
      if (i == selected) {
        g.setColor(Color.black);
      }
    }

    g.drawRect(0, 0, size.width - 1, size.height - 1);
  }

  public void setVisible(boolean visible) {
    if (!visible) {
      lookupWindow.setVisible(false);
      lookupWindow.dispose();
      lookupWindow = null;
    }
    super.setVisible(visible);
  }

  protected void processKeyEvent(KeyEvent event) {
    inputMethod.dispatchEvent(event);
  }
}
